//-----------------------------------------------------------------------------
// Copyright (c) 2001 Andreas Jnsson
//-----------------------------------------------------------------------------
#include "window.h"

//=============================================================================
CWindow::CWindow()
{
	hWnd = 0;
}

//=============================================================================
void CWindow::SetDefault(SParameters *pParams)
{
	pParams->hInstance  = 0;
	pParams->fullscreen = false;
	pParams->height     = 480;
	pParams->width      = 640;
	pParams->pMenu      = 0;
	pParams->pTitle     = "Window";
}

//=============================================================================
HRESULT CWindow::Initialize(SParameters &params)
{
	// Get a reference to the application for future use
	fullscreen  = params.fullscreen;

    // Register the window class
    WNDCLASS WndClass;
	WndClass.style         = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
	WndClass.lpfnWndProc   = (WNDPROC)WndProc;
	WndClass.cbClsExtra    = 0;
	WndClass.cbWndExtra    = 0;
	WndClass.hInstance     = params.hInstance;
	WndClass.hIcon         = LoadIcon(NULL, IDI_WINLOGO);
	WndClass.hCursor       = LoadCursor(NULL, IDC_ARROW);
	WndClass.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH); 
	WndClass.lpszMenuName  = params.pMenu;
	WndClass.lpszClassName = "Window";

    if( !RegisterClass( &WndClass ) )
	{
		return E_FAIL;
	}

    // Setup our window parameters
	int posX, posY, width, height;
	DWORD exStyle, style;
	if( params.fullscreen )
	{
		exStyle = WS_EX_TOPMOST; // This lets our window cover the whole screen
		style   = WS_POPUP;      // We don't need caption and system 
		                         // menu when in fullscreen mode

		// Set the window to cover the whole screen
		posX   = 0;
		posY   = 0;
		width  = GetSystemMetrics(SM_CXSCREEN);
		height = GetSystemMetrics(SM_CYSCREEN);
	}
	else
	{
		exStyle = 0;
		//style   = WS_OVERLAPPEDWINDOW; // This is a normal window
		style = WS_OVERLAPPED|WS_CAPTION|WS_SYSMENU|WS_MINIMIZEBOX|WS_DLGFRAME; 

		// Make sure the client area is the size we want
		RECT rect = {0, 0, params.width, params.height};
		AdjustWindowRect(&rect, style, params.pMenu != 0 ? TRUE : FALSE);

		posX   = CW_USEDEFAULT; // Let Windows position our window
		posY   = CW_USEDEFAULT;
		width  = rect.right - rect.left;
		height = rect.bottom - rect.top;
	}


    // Create the render window
	// Set lpParam to point to our application so that the window procedure
	// knows which class to call when handling window messages
    hWnd = CreateWindowEx( exStyle, "Window", 
		                   params.pTitle, style, posX, posY,
						   width,height,NULL,NULL,params.hInstance,this ); 
	if( hWnd == NULL )
	{
		return E_FAIL;
	}

	// Show the window
	ShowWindow( hWnd, SW_SHOW );
    UpdateWindow( hWnd );

	return S_OK;
}

//=============================================================================
// Static msg handler which passes messages to the application class.
LRESULT CALLBACK CWindow::WndProc( HWND hWnd, UINT msg, 
								   WPARAM wParam, LPARAM lParam )
{
	CWindow *pWindow;

	if( msg == WM_CREATE )
	{
		// Extract the window pointer parameter.
		CREATESTRUCT *pCS = (CREATESTRUCT *)lParam;
		pWindow = (CWindow *)pCS->lpCreateParams;

		if( pWindow )
		{
			// Save it in the window handle.
			SetWindowLong(hWnd, GWL_USERDATA, (LONG)pWindow);
		}
	}
	else
	{
		// Extract the object from the window handle.
		pWindow = (CWindow *)GetWindowLong(hWnd, GWL_USERDATA);
	}

	if( pWindow )
	{
		// Let the object handle it.
		return pWindow->MsgProc(hWnd, msg, wParam, lParam);
	}
	else
	{
		// Just do the default if the window is not created yet.
		return DefWindowProc(hWnd, msg, wParam, lParam);
	}
}


//=============================================================================
// If bWait is true then the function will wait until a message arrives.
// The function returns true if it receives WM_QUIT.
bool CWindow::CheckMessage(bool wait)
{
    MSG msg;
    
	if( wait )
		WaitMessage();

	while( PeekMessage(&msg, NULL, 0, 0, PM_REMOVE) )
    {
		if( msg.message == WM_QUIT )
			return true;

        TranslateMessage(&msg);
        DispatchMessage(&msg);
    }

    return false;
}

//=============================================================================
HWND CWindow::GetHandle()
{
	return hWnd;
}

//=============================================================================
void CWindow::GetClientRect(RECT *pRect)
{
	::GetClientRect(hWnd, pRect);
}

//=============================================================================
LRESULT CWindow::MsgProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
	// Remember the original parameters
	originalHWnd   = hWnd;
	originalMsg    = msg;
	originalWParam = wParam;
	originalLParam = lParam;

	switch( msg )
	{
	case WM_DESTROY:
		PostQuitMessage(0);
		return 0;
	}

	// Let the default window procedure handle 
	// any message that we don't care about
	return DefWindowProc(originalHWnd, originalMsg, 
		                 originalWParam, originalLParam);
}

